package com.alterdekim.game.message.serializer;

import com.alterdekim.game.message.SerializerUtils;
import com.alterdekim.game.utils.ByteUtils;
import lombok.extern.slf4j.Slf4j;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class BinaryMessageSerializer {
    public static List<Byte> serialize(Object message) {
        List<Field> fields = SerializerUtils.getPrivateFields(message.getClass());
        List<Byte> result = new ArrayList<>();
        fields.forEach(f -> {
            String type_name = ((Class) f.getType()).getName();
            try {
                switch (type_name) {
                    case "java.lang.String":
                        result.addAll(SerializerUtils.primitiveToBytes(((String) f.get(message)).getBytes()));
                        break;
                    case "java.lang.Integer":
                        result.addAll(serializeInteger((Integer) f.get(message)));
                        break;
                    case "java.lang.Boolean":
                        if( ((Boolean) f.get(message)).booleanValue() ) { result.add((byte) 1); } else { result.add((byte) 0); }
                        break;
                    case "java.lang.Long":
                        result.addAll(serializeLong((Long) f.get(message)));
                        break;
                    case "java.lang.Double":
                        result.addAll(serializeDouble((Double) f.get(message)));
                        break;
                    case "java.lang.Float":
                        result.addAll(serializeFloat((Float) f.get(message)));
                        break;
                    case "java.lang.Byte":
                        result.add((Byte) f.get(message));
                        break;
                    case "int":
                        result.addAll(serializeInteger(f.getInt(message)));
                        break;
                    case "boolean":
                        if( f.getBoolean(message) ) { result.add((byte) 1); } else { result.add((byte) 0); }
                        break;
                    case "long":
                        result.addAll(serializeLong(f.getLong(message)));
                        break;
                    case "double":
                        result.addAll(serializeDouble(f.getDouble(message)));
                        break;
                    case "float":
                        result.addAll(serializeFloat(f.getFloat(message)));
                        break;
                    case "byte":
                        result.add(f.getByte(message));
                        break;
                    default:
                        result.addAll(BinaryMessageSerializer.serialize(f.get(message)));
                        break;
                }
            } catch (IllegalAccessException e) {
                log.error(e.getMessage());
            }
        });
        return result;
    }

    public static Integer deserializeInteger(List<Byte> bytes) {
        assert bytes.size() == 4;
        int value = 0;
        for (int i = 0; i < bytes.size(); i++) {
            value = (value << 8) + (bytes.get(i) & 0xFF);
        }
        return value;
    }

    public static Integer deserializeInteger(byte[] bytes) {
        assert bytes.length == 4;
        int value = 0;
        for (int i = 0; i < bytes.length; i++) {
            value = (value << 8) + (bytes[i] & 0xFF);
        }
        return value;
    }

    public static Long deserializeLong(List<Byte> bytes) {
        assert bytes.size() == 8;
        long value = 0;
        for (int i = 0; i < bytes.size(); i++) {
            value = (value << 8) + (bytes.get(i) & 0xFF);
        }
        return value;
    }

    public static Long deserializeLongUnsafe(List<Byte> bytes) {
        long value = 0;
        for (int i = 0; i < bytes.size(); i++) {
            value = (value << 8) + (bytes.get(i) & 0xFF);
        }
        return value;
    }

    public static Long deserializeLong(byte[] bytes) {
        assert bytes.length == 8;
        long value = 0;
        for (int i = 0; i < bytes.length; i++) {
            value = (value << 8) + (bytes[i] & 0xFF);
        }
        return value;
    }

    private static List<Byte> serializeInteger(Integer i) {
        return serializeNumber(i.doubleValue());
    }

    private static List<Byte> serializeLong(Long l) {
        return serializeNumber(l.doubleValue());
    }

    private static List<Byte> serializeDouble(Double d) {
        return serializeNumber(d);
    }

    private static List<Byte> serializeFloat(Float f) {
        return serializeNumber(f.doubleValue());
    }

    private static List<Byte> serializeNumber(Double d) {
        return SerializerUtils.primitiveToBytes(ByteUtils.longToBytes(Double.doubleToLongBits(d)));
    }
}
